//
//-----------------------------------------------------------------------------------------------------------------
//
// SMSC SCH3114 Equates
//
// History:  
// When			Who				What 
//-------------------------------------------------------------
// 10/29/2007   Rick Stievenart	Genesis
// 11/09/2007	Rick Stievenart	Cleaned up prior to release.
//
//-----------------------------------------------------------------------------------------------------------------
//

#define SCH3114_DEVICE_ID				0x8C
#define SCH3114_COMPANY_ID				0x5C

#define STOPPED_FAN						0xffff
#define SLOW_FAN						0xfffe

#define SCH3114_RUNTIME_REGS			0x500	// I/O address of the "Runtime Register Set"


#define SCH3114_HWMON_INDEX_REG			0x570	// Hardware Monitor Logical Devices "Index" and "Data" Registers
#define SCH3114_HWMON_DATA_REG			0x571	// in the "Runtime Register Set"

//
// Hardware Monitor Register Set (Indexed Addresses)
//

#define SMSC_TEST						0x10    // SMSC Test Register

#define OFFSET_AMBIENT					0x1D	// Offset Register Ambient
#define OFFSET_D2						0x1E	// Offset Register 2
#define OFFSET_D1						0x1F	// Offset Register 1

#define V_2_5_READING					0x20	// +2.5V
#define V_1_5_READING					0x21	// From Vccp pin
#define V_VCC_READING					0x22	// VCC
#define V_5_READING						0x23	// +5V
#define V_12_READING					0x24	// +12V

#define D1_TEMP_READING					0x25	// Remote Diode 1 Temp Reading Register (MSB)
#define AMBIENT_TEMP_READING			0X26	// Ambient Temp Reading Register (MSB)
#define D2_TEMP_READING					0x27	// Remote Diode 2 Temp Reading Register (MSB)

#define FANTACH_1_LSB					0x28	// Fan Tach 1 LSB
#define FANTACH_1_MSB					0x29	// Fan Tach 1 MSB

#define FANTACH_2_LSB					0x2A	// Fan Tach 2 LSB
#define FANTACH_2_MSB					0x2B	// Fan Tach 2 MSB

#define FANTACH_3_LSB					0x2C	// Fan Tach 3 LSB
#define FANTACH_3_MSB					0x2D	// Fan Tach 3 MSB
										// 0x2e - Reserved
										// 0x2f - Reserved
#define PWM_1_CUR_DUTY					0x30  	// PWM1 Current Duty Cycle
#define PWM_2_CUR_DUTY					0x31  	// PWM2 Current Duty Cycle
#define PWM_3_CUR_DUTY					0x32  	// PWM3 Current Duty Cycle
										// 0x33-0x3c - Reserved
#define DEVICE_ID						0x3D	// Device Id
#define COMPANY_ID						0x3E	// Company Id
#define REVISION						0x3F	// Revision
										
#define READY_LOCK_START				0x40	// Ready, Lock, Start Register

// Bit Definition of bits in READY_LOCK_START Register
#define VBAT_MON					0x10	// Bit5 - VBAT Monitor
#define OVRID						0x08	// Bit4 - Override
#define READY						0x04	// Bit3 - Ready
#define LOCK						0x02	// Bit2 - Lock
#define START						0x01	// Bit1 - Start

#define INT_STAT_1						0x41	// Interrupt Status Register 1 (INT23, D2, AMB, D1, 5V, VCC, Vccp, 2.5V)

// Bit Definition of bits in INT_STAT_1 Register
#define IS1_INT23					0x80
#define IS1_D2						0x40
#define IS1_AMB						0x20
#define IS1_D1						0x10
#define IS1_5V						0x08
#define IS1_VCC						0x04
#define IS1_VCCP					0x02
#define IS1_2_5V					0x01

#define INT_STAT_2						0x42	// Interrupt Status Register 2 (ERR2, ERR1, RES, FANTACH1-3, RES, 12V)

// Bit Definition of bits in INT_STAT_2 Register
#define IS2_ERR2					0x80
#define IS2_ERR1					0x40
#define IS2_FANTACH3				0x10
#define IS2_FANTACH2				0x08
#define IS2_FANTACH1				0x04
#define IS2_12V						0x01

										// 0x43 - Reserved
#define V_2_5_LOW_LIMIT					0x44	// 2.5V Low Limit
#define V_2_5_HIGH_LIMIT				0x45	// 2.5V High Limit

#define V_VCCP_LOW_LIMIT				0x46	// VCCP Low Limit
#define V_VCCP_HIGH_LIMIT				0x47	// VCCP High Limit

#define V_VCC_LOW_LIMIT					0x48	// VCC Low Limit
#define V_VCC_HIGH_LIMIT				0x49	// VCC High Limit

#define V_5_LOW_LIMIT					0x4A	// +5V Low Limit
#define V_5_HIGH_LIMIT					0x4B	// +5V High Limit

#define V_12_LOW_LIMIT					0x4C	// +12V Low Limit
#define V_12_HIGH_LIMIT					0x4D	// +12V High Limit

#define D1_LOW_TEMP						0x4E	// Remote Diode 1 Low Temp
#define D1_HIGH_TEMP					0x4F	// Remote Diode 1 High Temp

#define AMBIENT_LOW_TEMP				0x50 	// Ambient Low Temp
#define AMBIENT_HIGH_TEMP				0x51	// Ambient High Temp

#define D2_LOW_TEMP						0x52	// Remote Diode 2 Low Temp
#define D2_HIGH_TEMP					0x53	// Remote Diode 2 High Temp

#define TACH_1_MIN_LSB					0x54	// Tach 1 Minimum
#define TACH_1_MIN_MSB					0x55

#define TACH_2_MIN_LSB					0x56	// Tach 2 Minimum
#define TACH_2_MIN_MSB					0x57

#define TACH_3_MIN_LSB					0x58	// Tach 3 Minimum
#define TACH_3_MIN_MSB					0x59
										// 0x5a-0x5b Reserved
#define PWM_1_CONFIG					0x5C	// PWM1 Configuration
#define PWM_2_CONFIG					0x5D	// PWM2 Configuration
#define PWM_3_CONFIG					0x5E	// PWM3 Configuration

// Bit Definition of bits in the PWM_x_CONFIG Registers
#define FAN_ON_ZONE_1_AUTO			0x00
#define FAN_ON_ZONE_2_AUTO			0x20
#define FAN_ON_ZONE_3_AUTO			0x40
#define FAN_ALWAYS_ON_FULL			0x60
#define FAN_DISABLED				0x80
#define FAN_HOTTEST_23				0xa0
#define FAN_HOTTEST_123				0xc0
#define FAN_MANUAL					0xe0

// Masks
#define ZONE_MASK					0xe0
#define SPIN_MASK					0x07

// Bit definitions 
#define INV							0x10
#define SUEN						0x08

#define SPIN_0MS					0x00
#define SPIN_100MS					0x01
#define SPIN_250MS					0x02
#define SPIN_400MS					0x03
#define SPIN_700MS					0x04
#define SPIN_1000MS					0x05
#define SPIN_2000MS					0x06
#define SPIN_4000MS					0x07

// RANGE/Freq
#define PWM_1_FREQ						0x5F	// Zone 1 Range/PWM1 Frequency
#define PWM_2_FREQ						0x60	// Zone 2 Range/PWM2 Frequency
#define PWM_3_FREQ						0x61	// Zone 3 Range/PWM3 Frequency

#define MIN_OFF_PWM_1_RAMP_RATE			0x62	// off/min controls, pwm1 ramp rate

#define OFF3						0x80		// off/min pwm3
#define OFF2						0x40		// off/min pwm2
#define OFF1						0x20		// off/min pwm1

#define RR1E						0x08		// PWM1 Ramp Rate enable
#define PWM1_RAMPRATE_MASK			0x07		// PWM1 Ramp Rate Mask

#define PWM_2_PWM_3_RAMP_RATE			0x63	// Pwm2/3 ramp rates

#define RR2E						0x80		// PWM2 Ramp Rate enable
#define PWM2_RAMPRATE_MASK			0x70		// PWM2 Ramp Rate Mask
#define PWM2_RAMPRATE_SHIFT			0x04

#define RR3E						0x08		// PWM3 Ramp Rate enable
#define PWM3_RAMPRATE_MASK			0x07		// PWM3 Ramp Rate Mask

#define RR_206MS					0x00
#define RR_104MS					0x01
#define RR_69MS						0x02
#define RR_41MS						0x03
#define RR_26MS						0x04
#define RR_18MS						0x05
#define RR_10MS						0x06
#define RR_5MS						0X07

#define PWM_1_MIN_DUTY					0x64	// PWM1 Minimum Duty Cycle
#define PWM_2_MIN_DUTY					0x65	// PWM2 Minimum Duty Cycle
#define PWM_3_MIN_DUTY					0x66	// PWM3 Minimum Duty Cycle

#define ZONE_1_LOW_TEMP_LIMIT			0x67	// Zone 1 Low Temperature Limit Register
#define ZONE_2_LOW_TEMP_LIMIT			0x68	// Zone 2 Low Temperature Limit Register
#define ZONE_3_LOW_TEMP_LIMIT			0x69	// Zone 3 Low Temperature Limit Register

#define ZONE_1_TEMP_ABS_LIMIT			0x6A	// Zone 1 Temperature Absolute Limit Register
#define ZONE_2_TEMP_ABS_LIMIT			0x6B	// Zone 2 Temperature Absolute Limit Register
#define ZONE_3_TEMP_ABS_LIMIT			0x6C	// Zone 3 Temperature Absolute Limit Register

#define ZONE_1_ZONE_2_HYST				0x6D	// Zone 1 and Zone 2 Hysteresis
#define ZONE_1_HYST_MASK			0xf0
#define ZONE_2_HYST_MASK			0x0f

#define ZONE_3_HYST						0x6E
#define ZONE_3_HYST_MASK			0xf0

										// 0x6f - Reserved
										// 0x70-0x79 - SMSC Test Register
                                        // 0x7A-0x7B - Reserved

#define SPECIAL_FUNCTION				0x7C	// Special Function Register

// Bit definitions in the Special Function Register
#define MONITOR_MD					0x02	// Bit 1 - Sets the monitoring mode - Continuous or Cycle
#define INTEN						0x04	// Bit 2 - Interrupt Enable bit
#define AVG_0						0x20	// Bit 5 - Average Bit 0
#define AVG_1						0x40	// Bit 6 - Average Bit 1
#define AVG_2						0x80	// Bit 7 - Average Bit 2

#define INT_ENAB_VOLTAGES				0x7E	// Interrupt Enable 1 (Voltages) Register

// Bit Definitions in the Interrupt Enable 1 (Voltages) Register
#define V_VCC_INT_EN				0x80	// Bit8 - Vccp Interrupt Enable
#define V_12_INT_EN					0x40	// Bit7 - +2.5V Interrupt Enable
#define V_5_INT_EN					0x20	// Bit6 - VBAT Interrupt Enable
#define V_VTR_INT_EN				0x10	// Bit5 - Voltage Interrupt Enable (Group)
#define V_VCCP_INT_EN				0x08	// Bit4 - Vccp Interrupt Enable
#define V_2_5_INT_EN				0x04	// Bit3 - +2.5V Interrupt Enable
#define V_VBAT_INT_EN				0x02	// Bit2 - VBAT Interrupt Enable
#define VOLT_INT_EN					0x01	// Bit1 - Voltage Interrupt Enable (Group)

#define CONFIGURATION					0x7F	// Configuration Register

// Bit definitions in the Configuration Register
#define CF_INIT						0x80	// Bit 7 - Set to perform Soft Reset of the Hardware Monitoring registers
											// except the reading registers. This bit is self-clearing.

#define CF_SUREN					0x10	// Spin-up reduction enable
#define CF_TRDY						0x08
#define CF_MON_DN					0x04
                                   
#define INT_ENAB_FAN_TACHS				0x80	// Interrupt Enable (Fan Tachs) Register

// Bit Definitions in the Interrupt Enable (Fan Tachs) Register
#define FANTACH_3_INT_EN			0x08	// Bit4 - Fan Tach 3 Interrupt Enable
#define FANTACH_2_INT_EN			0x04	// Bit3 - Fan Tach 2 Interrupt Enable
#define FANTACH_1_INT_EN			0x02	// Bit2 - Fan Tach 1 Interrupt Enable
#define FANTACH_INT_EN				0x01	// Bit1 - Fan Tach Interrupt Enable (Group)

#define TACH_PWM_ASSOC					0x81	// Tach PWM Association

#define TACH3_PWM_ASSOC_MASK  	0x30
#define TACH2_PWM_ASSOC_MASK    0x0C
#define TACH1_PWM_ASSOC_MASK 	0x03
#define TACH3_PWM_ASSOC_SHIFT	4
#define TACH2_PWM_ASSOC_SHIFT	2


// Bit Definitions in the TACH_PWM_ASSOC Register
#define T3H							0x20	// Bit6
#define T3L							0x10	// Bit5
#define T2H							0x08	// Bit4
#define T2L							0x04	// Bit3
#define T1H							0x02	// Bit2
#define T1L							0x01	// Bit1

#define INT_ENAB_TEMP					0x82	// Interrupt Enable (Temp) Register

// Bit Definitions in the Interrupt Enable (Temp) Register
#define D2_INT_EN					0x08	// Bit4 - Remote Diode 2 Interrupt Enable
#define D1_INT_EN					0x04	// Bit3 - Remote Diode 1 Interrupt Enable
#define AMBIENT_INT_EN				0x02	// Bit2 - Ambient Interrupt Enable
#define TEMP_INT_EN					0x01	// Bit1 - Temperature Interrupt Enable (Group)

#define INT_STAT_3						0x83	// Interrupt Status (Voltages 2) Register 3

// Bit Definitions in the Interrupt Status (Voltage 2) Register 3
#define IS3_VBAT					0x02
#define IS3_VTR						0x01

// A/D Convertor LSBs
#define AD_LSB_REG_5					0x84
#define AD_LSB_REG_1					0x85
#define AD_LSB_REG_2					0x86
#define AD_LSB_REG_3					0x87
#define AD_LSB_REG_4					0x88
										// 0x89 - Reserved
										// 0x8a-0x8e - SMSC Test Register
										// 0x8f - Reserved
#define FANTACH_1_OPTION				0x90	// FanTach 1 Option Register
#define FANTACH_2_OPTION				0x91	// FanTach 2 Option Register
#define FANTACH_3_OPTION				0x92	// FanTach 3 Option Register

#define FT_3EDG					0x10
#define FT_MODE					0x08
#define FT_EDG1					0x04
#define FT_EDG0					0x02
#define FT_SLOW					0x01
										// 0x93 - Reserved
#define PWM_1_OPTION					0x94	// PWM 1 Option Register
#define PWM_2_OPTION					0x95	// PWM 2 Option Register
#define PWM_3_OPTION					0x96	// PWM 3 Option Register

// Bit definitions of bits in PWM_x_OPTION Registers
#define OPP						0x20
#define	GRD1					0x10
#define GRD0					0x08
#define SZEN					0x04
#define UPDT1					0x02
#define UPDT0					0x01

										// 0x97-0x98 - SMSC Test Registers

#define VTR_READING						0x99	// VTR Reading Register
#define VBAT_READING					0x9A	// VBAT Reading Register

#define VTR_LIMIT_LOW					0x9B	// VTR Limit Low
#define VTR_LIMIT_HI					0x9C	// VTR Limit High

#define VBAT_LIMIT_LOW					0x9D	// VBAT Limit Low
#define VBAT_LIMIT_HI					0x9E	// VBAT Limit High
										// 0x9F-0xA2 - Reserved
                                        // 0xA3-0xA4 - SMSC Test Registers

#define INT_STAT_1_SEC					0xA5	// Interrupt Status Register 1 Secondary
#define INT_STAT_2_SEC					0xA6	// Interrupt Status Register 2 Secondary
#define INT_STAT_3_SEC					0xA7	// Interrupt Status Register 3 Secondary

										// 0xA8 - Reserved
										// 0xA9-0xAA - SMSC Test Register

#define TACH_1_3_MODE					0xAB

										// 0xAC - Reserved
										// 0xAD - SMSC Test Register

#define TOP_TEMP_REMOTE_DIODE_1			0xAE	// Top Temperature Remote Diode 1 (Zone 1) Register
#define TOP_TEMP_REMOTE_DIODE_2			0xAF	// Top Temperature Remote Diode 2 (Zone 3) Register

										// 0xB0 - 0xB2 SMSC Test Registers

#define TOP_TEMP_AMBIENT				0xB3	// Top Temperature Ambient (Zone 2) Register

#define MIN_TEMP_ADJ_TEMP_RD1_RD2		0xB4	// Min tempadj
#define MIN_TEMP_ADJ_TEMP_DLY_AMB		0xB5

#define MIN_TEMP_ADJ_DLY_RD1_RD2		0xB6

#define TRIM_ADJ_EN						0xB7

#define TOP_TEMP_EXCEEDED_STAT			0xB8

        								// 0xB9 - Reserved
        								// 0xBA-0xBB - SMSC Reserved
        								// 0xBC - Reserved
        								// 0xBD - SMSC Reserved
        								// 0xBE - Reserved
        								// 0xBF-0xC0 SMSC Reserved

#define THERMTRIP_CNTL					0xC1
										// 0xC2-0xC3 Reserved

#define THERMTRIP_TEMP_LIMIT_D1			0xC4	// THERMTRIP Temperature Limit Remote Diode 1 (Zone 1) Register
#define THERMTRIP_TEMP_LIMIT_D2			0xC5	// THERMTRIP Temperature Limit Remote Diode 2 (Zone 3) Register
										// 0xC6-0xC8 - Reserved
#define THERMTRIP_TEMP_LIMIT_AMBIENT	0xC9	// THERMTRIP Temperature Limit Ambient (Zone 2) Register

#define THERMTRIP_STAT					0xCA

#define THERMTRIP_OUT_EN				0xCB

// Bit Definitions in the ThermTrip Status, Top Temp Exceeded Status and ThermTrip Output Enable Registers
#define AMBIENT_TRIP				0x01
#define D1_TRIP						0X02
#define D2_TRIP						0x04

										// 0xCC-0xCD - Reserved

#define THERMTRIP_ENABLE				0xCE	// THERMTRIP Enable Register (????)

										// 0xCF-0xD0 SMSC Test Register
#define PWM_1_MAX_DUTY					0xD1
										// 0xD2-0xD5 Reserved
#define PWM_2_MAX_DUTY					0xD6
										// 0xD7-0xDA Reserved
#define PWM_3_MAX_DUTY					0xDB
										// 0xDC-0xDF Reserved
#define EN_LSBS_FOR_AUTOFAN				0xE0
										// 0xE1-0xE8 Reserved
										// 0xE9-0xEE SMSC Reserved
										// 0xEF-0xFe (????)
										// 0xFF - SMSC Test Register
