//
//-----------------------------------------------------------------------------------------------------------------
//
// hwmon - EBC-855 SMC3114 Hardware Monitor Tester and Sample Application
//
// History:  
// When			Who				What 
//-----------------------------------------------------------------------------------------------------------------
// 10/29/2007   Rick Stievenart	Genesis
// 11/09/2007	Rick Stievenart	Cleaned up prior to release.
// 11/16/2007	Rick Stievenart	Major overhaul complete
// 11/29/2007	Rick Stievenart	Released in executable and source forms.
//
//-----------------------------------------------------------------------------------------------------------------
//

//#define DEBUG 1
//#define DEBUG_VIDEO 1

#include <stdio.h>
#include <stdlib.h> 
#include <conio.h>
#include <process.h>
#include <string.h>
#include <conio.h>

#include <graph.h>

#include "winsys.h"
#include "sch3114.h"

// This is who we are
struct winsystems_identification self = {
	BETA_RELEASE,
	1,
	0
};

// Function return value definitions
#define SUCCESS	0
#define FAILURE 1

// Function definitions
void signon(void);

void usage(char *progname);

int hwmon_probe(void);
int hwmon_init(void);

void register_dump(void);
void config_dump(void);

unsigned short show_fanmode(unsigned char fanmode, unsigned short c);
unsigned short show_tach_pwm(unsigned char pwm, unsigned short c);
unsigned short show_spin(unsigned char spin, unsigned short c);
unsigned short show_ramp_rate(unsigned char spin, unsigned short c);

void statmon(void);
void tempmon(void);
void voltmon(void);
void fanmon(void);

unsigned int read_fantach(int fan);

unsigned char hmReadReg(unsigned char reg);
void hmWriteReg(unsigned char reg, unsigned char val);

// Global Variables
char buff[2048];

int show_statuses;
int show_registers;
int show_config;
int show_temperatures;
int show_voltages;
int show_fans;
int continuous;
int verbose;
int reset_fault_statuses;
int user_init;
int default_init;
int override;
int stop;
int running;
int normal;

int min_pwm_duty;

struct _videoconfig vc;

//
//--------------------------------------------------------------
// Main - start of it all
//--------------------------------------------------------------
//

int main(int argc, char *argv[], char *envp[])
{
	short oldmode, oldfgd, c;
	long  oldbgd;
	struct _rccoord oldpos;
   	int i, retval = SUCCESS;
	unsigned char temp;

	// Set defaults
	show_registers =		FALSE;
	show_config = 			FALSE;
	continuous =	 		FALSE;
	verbose = 				FALSE;
	user_init = 			FALSE;
	default_init = 			FALSE;
	show_statuses = 		TRUE;
	show_temperatures =		TRUE;
	show_voltages =			TRUE;
	show_fans = 			TRUE;
    reset_fault_statuses =	FALSE;
	override = 				FALSE;
	stop = 					FALSE;
	running = 				TRUE;
	normal = 				TRUE;

	min_pwm_duty =			0x00;				// minimum pwm duty cycle

	// Check for arguments
	if(argc < 1) {
		// Incorrect number of arguments, print usage info
		usage(argv[0]);
		exit(FAILURE);
	}

	// Process each command line argument
	for(i = 1; i < argc; i++) {  // Skip argv[0] (program name)
		if(!strcmp(argv[i], "-?")) {	// Print usage information
			usage(argv[0]);
			exit(FAILURE);

		} else if(!strcmp(argv[i], "-r")) {	// check for "Show Registers" command line argument
			show_registers = TRUE;

 		} else if(!strcmp(argv[i], "-f")) {	// check for "Show Configuration" command line argument
			show_config = TRUE;
 
 		} else if(!strcmp(argv[i], "-c")) {	// check for "Continuous" (operation modifier) command line argument
 			continuous = TRUE; 

 		} else if(!strcmp(argv[i], "-u")) {	// check for "User Initialization" (operation modifier) command line argument
 			user_init = TRUE; 

 		} else if(!strcmp(argv[i], "-d")) {	// check for "Default Initialization" (operation modifier) command line argument
 			default_init = TRUE; 
 
 		} else if(!strcmp(argv[i], "-v")) {	// check for "Verbose" (operation modifier) command line argument
 			verbose = TRUE; 

 		} else if(!strcmp(argv[i], "-m")) {	// check for "minimum pwm duty cycle" command line argument
			int tmp;
			
            i++;
 
 			tmp = sscanf(argv[i], "%d", &min_pwm_duty);
			if(tmp == 0 || tmp == EOF) {
            	// usage
	 			usage(argv[0]);
				exit(FAILURE);
			}

 		} else {
			fprintf(stderr, "Unrecognized command line option: %s\n", argv[i]);	// Don't know this one, tell 'em and bail
			exit(FAILURE);
   		}
 	}

	// Everythings good to go, see if we have an SMC3114 were it is supposed to be
	if(!hwmon_probe()) {
		fprintf(stderr, "SCH3114 not present at 0x%x.\n", SCH3114_HWMON_INDEX_REG);
		exit(FAILURE);
    }
 
	// Save original foreground, background, and text position.
	oldfgd = _gettextcolor();
	oldbgd = _getbkcolor();
	oldpos = _gettextposition();
	_getvideoconfig(&vc);
    oldmode = vc.mode;
 
 #ifdef DEBUG_VIDEO
   /* Write all information to a string, then output string. */
   c  = sprintf(buff,     "X pixels:     %d\n", vc.numxpixels);
   c += sprintf(buff + c, "Y pixels:     %d\n", vc.numypixels);
   c += sprintf(buff + c, "Text columns: %d\n", vc.numtextcols);
   c += sprintf(buff + c, "Text rows:    %d\n", vc.numtextrows);
   c += sprintf(buff + c, "Colors:       %d\n", vc.numcolors);
   c += sprintf(buff + c, "Bits/pixel:   %d\n", vc.bitsperpixel);
   c += sprintf(buff + c, "Video pages:  %d\n", vc.numvideopages);
   c += sprintf(buff + c, "Mode:         %d\n", vc.mode);
   c += sprintf(buff + c, "Adapter:      %d\n", vc.adapter);
   c += sprintf(buff + c, "Monitor:      %d\n", vc.monitor);
   c += sprintf(buff + c, "Memory:       %d\n", vc.memory);
   _outtext(buff);

	_getch();
	_clearscreen(_GCLEARSCREEN);	// Clear the screen
 	_settextposition(1, 1);			// Start at the top left corner
#endif
    
	// Find a valid graphics mode, the higher the better
	if(!_setvideomode(_MAXRESMODE)) {
		return(FAILURE);
	}

 	_setbkcolor(0);					// Set background color to black
	_settextcolor(7);				// Set foregound color to white
	_clearscreen(_GCLEARSCREEN);	// Clear the screen
 	_settextposition(1, 1);			// Start at the top left corner
	_getvideoconfig(&vc);

#ifdef DEBUG_VIDEO
   /* Write all information to a string, then output string. */
   c  = sprintf(buff,     "X pixels:     %d\n", vc.numxpixels);
   c += sprintf(buff + c, "Y pixels:     %d\n", vc.numypixels);
   c += sprintf(buff + c, "Text columns: %d\n", vc.numtextcols);
   c += sprintf(buff + c, "Text rows:    %d\n", vc.numtextrows);
   c += sprintf(buff + c, "Colors:       %d\n", vc.numcolors);
   c += sprintf(buff + c, "Bits/pixel:   %d\n", vc.bitsperpixel);
   c += sprintf(buff + c, "Video pages:  %d\n", vc.numvideopages);
   c += sprintf(buff + c, "Mode:         %d\n", vc.mode);
   c += sprintf(buff + c, "Adapter:      %d\n", vc.adapter);
   c += sprintf(buff + c, "Monitor:      %d\n", vc.monitor);
   c += sprintf(buff + c, "Memory:       %d\n", vc.memory);
   _outtext(buff);

	_getch();
	_clearscreen(_GCLEARSCREEN);	// Clear the screen
 	_settextposition(1, 1);			// Start at the top left corner
#endif

	// Print the signon banner
	signon();
	
   
	// Do they want to see the registers?
	if(show_registers) {

		register_dump();

	} else {
	
 		hwmon_init();

		if(show_config) {

			config_dump();

 //----------------------------------------------------------------------+----------------------------------------------------------------

			_settextposition((vc.numtextrows - 1), ((vc.numtextcols / 2) - (strlen("           <Press any key to continue>           ") / 2)));
			_outtext("           <Press any key to continue>           ");
			_getch();
 
			c = 0;
			_clearscreen(_GCLEARSCREEN);	// Clear the screen
			_settextposition(1, 1);			// Start at the top left corner

//----------------------------------------------------------------------+----------------------------------------------------------------

			signon();
		}

		do {
do_it_all_again:

			_settextposition(6, 1);

			if(normal && override) {

				temp = hmReadReg(READY_LOCK_START);		// Set override to force fans to 100%
				temp |= OVRID;
				hmWriteReg(READY_LOCK_START, temp);
				normal = FALSE;

			}
			if(!normal && !override) {

				temp = hmReadReg(READY_LOCK_START);		// Clear override
				temp &= ~OVRID;
				hmWriteReg(READY_LOCK_START, temp);
				normal = TRUE;

			}

			if(running && stop) {

				temp = hmReadReg(READY_LOCK_START);		// Stop requested?
				temp &= ~START;							// Clear START
				hmWriteReg(READY_LOCK_START, temp);

#ifdef DEBUG	
				printf("Stopping...");
				_outp(0x80, 0xfe);
#endif
		        // poll for monitoring cycle done
				while(!(hmReadReg(CONFIGURATION) & CF_MON_DN)) {
					;
				}				
#ifdef DEBUG
				printf("Stopped\n");
				_outp(0x80, 0xff);
#endif
                running = FALSE;

			}

			if(!running && !stop) {

#ifdef DEBUG
				printf("Starting...");
				_outp(0x80, 0x80);
#endif
				temp = hmReadReg(READY_LOCK_START);
				temp |= START;
				hmWriteReg(READY_LOCK_START, temp);
				
		        // poll for voltages and temperatures ready
				while(!(hmReadReg(CONFIGURATION) & CF_TRDY)) {
					;
				}				
#ifdef DEBUG
				printf("Started\n");
				_outp(0x80, 0x81);
#endif
				running = TRUE;
			}
			
			// Do they want to see statuses?	
			if(show_statuses) {
				statmon();
			}
			
			// Do they want to see temperatures?	
			if(show_temperatures) {
				tempmon();
			}

			// Do they want to see voltages?
			if(show_voltages) {
				voltmon();
			}

			// Do they want to see fan tachs
			if(show_fans) {
				fanmon();
			}
 
 			if(continuous) {
 				_settextposition((vc.numtextrows - 1), ((vc.numtextcols / 2) - (strlen("<Press any key to exit, 'r' to reset faults>") / 2)));
 				_outtext("<Press any key to exit, 'r' to reset faults>");
 			}
 
 			reset_fault_statuses = FALSE;
 			
 		} while(continuous && !_kbhit());

 		if(continuous) {
 			c = _getch();	// from the kbhit keystroke
 			if(tolower(c) == 'r') {
 			
 				reset_fault_statuses = TRUE;
				goto do_it_all_again;

 			} else if(tolower(c) == 'o') {

 				override = TRUE;
 				goto do_it_all_again;

   			} else if(tolower(c) == ' ') {

  				override = FALSE;
 				goto do_it_all_again;

   			}  else if(tolower(c) == 's') {

  				stop = TRUE;
 				goto do_it_all_again;

   			}  else if(tolower(c) == 't') {

  				stop = FALSE;
 				goto do_it_all_again;

			}
   		}
 	}

    _settextposition((vc.numtextrows - 1), ((vc.numtextcols / 2) - (strlen("           <Press any key to exit>           ") / 2)));
	_outtext("           <Press any key to exit>           ");
	_getch();
 	
 	// Restore original foreground, background, and text position.
 	if(!_setvideomode(oldmode)) {
		exit(retval);
	}
	_settextcolor(oldfgd);
	_setbkcolor(oldbgd);
 
	// We are out of here
	exit(retval);
}

//
//--------------------------------------------------------------
// signon - Print signon
//--------------------------------------------------------------
//

void signon(void)
{
	short c;
	
	// Print a Signon Banner
	c = sprintf(buff, "WinSystems EBC-855 SMC3114 Hardware Monitor Tester\n");
	c += sprintf(buff + c, "Version: ");
	c += sprintf(buff + c, "%02d.%02d ", self.major, self.minor);
	c += sprintf(buff + c, __TIMESTAMP__);
	c += sprintf(buff + c, "\n");
 	c += sprintf(buff + c, "%s\n", OUR_COPYRIGHT);
	if(self.rt == ALPHA_RELEASE) {
		c += sprintf(buff + c, "************** ALPHA RELEASE - FOR INTERNAL USE ONLY ***************\n\n");
	} else if(self.rt == BETA_RELEASE) {
		c += sprintf(buff + c, "*************** BETA RELEASE - FOR TESTING PURPOSES ONLY ***********\n\n");
	} else if(self.rt == ENGINEERING_RELEASE) {
		c += sprintf(buff + c, "*** ENGINEERING RELEASE - FOR INTERNAL ENGINEERING PURPOSES ONLY ***\n\n");
	} else {
		c += sprintf(buff + c, "\n");
	}
	_outtext(buff);
}

//
//--------------------------------------------------------------
// Usage - Print purpose and usage information
//--------------------------------------------------------------
//

void usage(char *progname)
{
	char *name, *name_end;
	
	name = strrchr(progname, '\\');
	if(name == NULL) {
		name = progname;
	} else {
		// point at first letter past the last '\'
		name++;
	}
	name_end = strrchr(progname, '.');
	if(name_end != NULL) {
		// cut off the filename at the '.'
		*name_end = '\0';
	}
	fprintf(stderr, "PURPOSE: Tests EBC-855 SMC3114 Hardware Monitor\n\n");
	fprintf(stderr, "USAGE: %s [-?] [-r] [-c] [-v] [-u] [-d] [-m xx] [-f]\n", name);
	fprintf(stderr, "  where: \n"); 
	fprintf(stderr, "%s -r: Prints SMC3114 Hardware Monitor Register Set\n", name);	
	fprintf(stderr, "%s -c: Modifies operation to print values continuously\n", name);
	fprintf(stderr, "%s -v: Modifies operation to print extra information\n", name);	
	fprintf(stderr, "%s -u: Reinitializes the SMC3114 with user values\n", name);	
	fprintf(stderr, "%s -d: Reinitializes the SMC3114 with default values\n", name);	
	fprintf(stderr, "%s -m xx: Sets PWM minimum Duty Cycle to xx\n", name);	
	fprintf(stderr, "%s -f: Prints the SMC3114 configuration\n", name);	
	fprintf(stderr, "%s -?: Prints this information\n\n", name);	
}


//
//-----------------------------------------------------------------
//
// hwmon_probe - Probe and verify that an SMC3114 is present,
//
//			The BIOS has already initialized the part - 
//			The resultant state of the part is:
// 			Limits set, fan in auto mode, interrupt masks
//			set, power management set and the part is monitoring.
//
//-----------------------------------------------------------------
//

int hwmon_probe(void)
{
	unsigned char device_id, company_id;

	// Read the SCH3114 Device ID Register 
	device_id = hmReadReg(DEVICE_ID);

	// Read the SCH3114 Company ID Register	
	company_id = hmReadReg(COMPANY_ID);

	// Check Company and Device ID's to see if they match what we expect, if not bail out with error
	if((device_id != SCH3114_DEVICE_ID) && (company_id != SCH3114_COMPANY_ID)) {
		return(0);
	}
	return(1);
}

//
//-----------------------------------------------------------------
//
// hwmon_init -	perform any initialization necessary
// 			prior to active monitoring.
//
//-----------------------------------------------------------------
//

int hwmon_init(void)
{
	unsigned char x;

	//
    // Is user_init option set?
	//
	if(user_init || default_init) {
		//
		// Yes, set our values...
		//	
#ifdef DEBUG
		printf("hwmon_init: stopping...");
		_outp(0x80, 0x40);
#endif

		x = hmReadReg(READY_LOCK_START);
		x &= ~START;
		hmWriteReg(READY_LOCK_START, x);						// stop everything

        // poll for monitoring done
		while(!(hmReadReg(CONFIGURATION) & CF_MON_DN)) {
			;
		}
		
#ifdef DEBUG
		printf("hwmon_init: stopped...");
		_outp(0x80, 0x41);
#endif
	
        running = FALSE;

		x = hmReadReg(CONFIGURATION);							// set the init bit to do a "soft reset" of the hm block
		x |= CF_INIT;
		hmWriteReg(CONFIGURATION, x);

#ifdef DEBUG
		printf("hwmon_init: initing...");
		_outp(0x80, 0x42);
#endif

		while(hmReadReg(CONFIGURATION) & CF_INIT) {				// wait for init to clear
			;
		}

		if(user_init) { 
#ifdef DEBUG
			printf("hwmon_init: user_init...");
			_outp(0x80, 0x43);
#endif
		
	 		// Set the fan tach low limit registers
  			hmWriteReg(TACH_1_MIN_LSB, 0xfd);
			hmWriteReg(TACH_1_MIN_MSB, 0xff);
  			hmWriteReg(TACH_2_MIN_LSB, 0xff);
	 		hmWriteReg(TACH_2_MIN_MSB, 0xff);
		  	hmWriteReg(TACH_3_MIN_LSB, 0xff);
  			hmWriteReg(TACH_3_MIN_MSB, 0xff);
  
 			// Set the top temp registers
  			hmWriteReg(TOP_TEMP_REMOTE_DIODE_1, 0x5f);
			hmWriteReg(TOP_TEMP_REMOTE_DIODE_2, 0x5f);
   			hmWriteReg(TOP_TEMP_AMBIENT, 0x5f);
 
 	 		// Set the zone low temp limit registers
  			hmWriteReg(ZONE_1_LOW_TEMP_LIMIT, 0x27);
 			hmWriteReg(ZONE_2_LOW_TEMP_LIMIT, 0x27);
  			hmWriteReg(ZONE_3_LOW_TEMP_LIMIT, 0x27);
 
	 		// Set the zone temp abs limit registers
  			hmWriteReg(ZONE_1_TEMP_ABS_LIMIT, 0x80);
 			hmWriteReg(ZONE_2_TEMP_ABS_LIMIT, 0x80);
  			hmWriteReg(ZONE_3_TEMP_ABS_LIMIT, 0x80);
  		
			// Set the hysteresis regs
  			hmWriteReg(ZONE_1_ZONE_2_HYST, 0x11);
  			hmWriteReg(ZONE_3_HYST, 0x10);
 
	  		// Set the thermtrip temp limit registers
  			hmWriteReg(THERMTRIP_TEMP_LIMIT_D1, 0x64);
 			hmWriteReg(THERMTRIP_TEMP_LIMIT_D2, 0x64);
  			hmWriteReg(THERMTRIP_TEMP_LIMIT_AMBIENT, 0x64);
  		
			// Set the PWM minimum duty cycle
  			hmWriteReg(PWM_1_MIN_DUTY, min_pwm_duty);
			hmWriteReg(PWM_2_MIN_DUTY, min_pwm_duty);
  			hmWriteReg(PWM_3_MIN_DUTY, min_pwm_duty);
                                        
			hmWriteReg(PWM_1_MAX_DUTY, 0xff);
  			hmWriteReg(PWM_2_MAX_DUTY, 0xff);
  			hmWriteReg(PWM_3_MAX_DUTY, 0xff);

			// Enable Statuses and Interrupts
			hmWriteReg(INT_ENAB_FAN_TACHS,	(FANTACH_1_INT_EN | FANTACH_INT_EN));	// enable fantach1 to set status register bits
                   
			hmWriteReg(INT_ENAB_TEMP, (D1_INT_EN | TEMP_INT_EN));					// enable d1 temp to set status register bits

			hmWriteReg(INT_ENAB_VOLTAGES, (V_VCC_INT_EN	| V_12_INT_EN	|
											  V_5_INT_EN	| V_VTR_INT_EN	|
											  V_VCCP_INT_EN | V_2_5_INT_EN	|
											  V_VBAT_INT_EN | VOLT_INT_EN));							// enable all voltages to set status register bits

	 		hmWriteReg(PWM_1_CONFIG, (FAN_ON_ZONE_1_AUTO | SPIN_250MS | SUEN));		// PWM_1_CONFIG = Fan 1 auto
 			hmWriteReg(PWM_2_CONFIG, (FAN_DISABLED | SPIN_0MS | SUEN));		// PWM_2_CONFIG = disabled
 			hmWriteReg(PWM_3_CONFIG, (FAN_DISABLED | SPIN_0MS | SUEN));		// PWM_3_CONFIG = disabled
 
 			hmWriteReg(PWM_1_FREQ, 0x70);								 	// Range and frequency, 11.0hz
 			hmWriteReg(PWM_2_FREQ, 0x70);								 	// Range and frequency, 11.0hz
 			hmWriteReg(PWM_3_FREQ, 0x70);								 	// Range and frequency, 11.0hz
 
			hmWriteReg(MIN_OFF_PWM_1_RAMP_RATE, 0x00);
 			hmWriteReg(PWM_2_PWM_3_RAMP_RATE, 0x00);
 
	   		hmWriteReg(TACH_PWM_ASSOC, 0x24);								// PWM3-TACH3, PWM2-TACH2, PWM1-TACH1
 
	  		hmWriteReg(PWM_1_OPTION, (OPP | GRD0 | UPDT1 | SZEN));
	  		hmWriteReg(PWM_2_OPTION, (OPP | GRD0 | UPDT1 | SZEN));
	  		hmWriteReg(PWM_3_OPTION, (OPP | GRD0 | UPDT1 | SZEN));
 
	   		hmWriteReg(FANTACH_1_OPTION, (FT_MODE | FT_EDG1 | FT_SLOW));
	   		hmWriteReg(FANTACH_2_OPTION, (FT_MODE | FT_EDG1 | FT_SLOW));
	   		hmWriteReg(FANTACH_3_OPTION, (FT_MODE | FT_EDG1 | FT_SLOW));
  		
	   		hmWriteReg(TACH_1_3_MODE, 0); 									// normal mode, no locked rotor mode

			hmWriteReg(EN_LSBS_FOR_AUTOFAN, 0);
 
			hmWriteReg(THERMTRIP_OUT_EN, D1_TRIP);							// Thermal trip from D1 only
			hmWriteReg(THERMTRIP_ENABLE, D1_TRIP);	
			hmWriteReg(THERMTRIP_CNTL, 0x00);								// No thermal trip output 
		
			hmWriteReg(SPECIAL_FUNCTION, INTEN);							// Interrupts enabled

			hmWriteReg(CONFIGURATION, 0x00); 							// Spin up reduction disabled

		}
		
		if(default_init) {
			// default init
#ifdef DEBUG
			printf("hwmon_init: default_init...");
			_outp(0x80, 0x43);
#endif
	 		// Set the fan tach low limit registers
  			hmWriteReg(TACH_1_MIN_LSB, 0xff);
			hmWriteReg(TACH_1_MIN_MSB, 0xff);
  			hmWriteReg(TACH_2_MIN_LSB, 0xff);
	 		hmWriteReg(TACH_2_MIN_MSB, 0xff);
		  	hmWriteReg(TACH_3_MIN_LSB, 0xff);
  			hmWriteReg(TACH_3_MIN_MSB, 0xff);
  
 			// Set the top temp registers
  			hmWriteReg(TOP_TEMP_REMOTE_DIODE_1, 0x2d);
			hmWriteReg(TOP_TEMP_REMOTE_DIODE_2, 0x2d);
   			hmWriteReg(TOP_TEMP_AMBIENT, 0x2d);
 
 	 		// Set the zone low temp limit registers
  			hmWriteReg(ZONE_1_LOW_TEMP_LIMIT, 0x27);
 			hmWriteReg(ZONE_2_LOW_TEMP_LIMIT, 0x27);
  			hmWriteReg(ZONE_3_LOW_TEMP_LIMIT, 0x27);
 
	 		// Set the zone temp abs limit registers
  			hmWriteReg(ZONE_1_TEMP_ABS_LIMIT, 0x5f);
 			hmWriteReg(ZONE_2_TEMP_ABS_LIMIT, 0x64);
  			hmWriteReg(ZONE_3_TEMP_ABS_LIMIT, 0x64);
  		
			// Set the hysteresis regs
  			hmWriteReg(ZONE_1_ZONE_2_HYST, 0x44);
  			hmWriteReg(ZONE_3_HYST, 0x40);
 
	  		// Set the thermtrip temp limit registers
  			hmWriteReg(THERMTRIP_TEMP_LIMIT_D1, 0x7f);
 			hmWriteReg(THERMTRIP_TEMP_LIMIT_D2, 0x7f);
  			hmWriteReg(THERMTRIP_TEMP_LIMIT_AMBIENT, 0x7f);
  		
			// Set the PWM minimum duty cycle
  			hmWriteReg(PWM_1_MIN_DUTY, min_pwm_duty);
			hmWriteReg(PWM_2_MIN_DUTY, min_pwm_duty);
  			hmWriteReg(PWM_3_MIN_DUTY, min_pwm_duty);
                                        
			hmWriteReg(PWM_1_MAX_DUTY, 0xff);
  			hmWriteReg(PWM_2_MAX_DUTY, 0xff);
  			hmWriteReg(PWM_3_MAX_DUTY, 0xff);

			// Enable Statuses and Interrupts
			hmWriteReg(INT_ENAB_FAN_TACHS,	0x00);							// disable all fantachs from setting status register bits
                   
			hmWriteReg(INT_ENAB_TEMP, (D1_INT_EN | TEMP_INT_EN));			// enable d1 temp to set status register bits

			hmWriteReg(INT_ENAB_VOLTAGES, 0x00);							// disable all voltages

	 		hmWriteReg(PWM_1_CONFIG, (FAN_ON_ZONE_1_AUTO | SPIN_1000MS | SUEN));		// PWM_1_CONFIG = Fan 1 auto
 			hmWriteReg(PWM_2_CONFIG, (FAN_DISABLED | SPIN_0MS));			// PWM_2_CONFIG = disabled
 			hmWriteReg(PWM_3_CONFIG, (FAN_DISABLED | SPIN_0MS));			// PWM_3_CONFIG = disabled
 
 			hmWriteReg(PWM_1_FREQ, 0x7b);								 	// Range and frequency, 11.0hz
 			hmWriteReg(PWM_2_FREQ, 0xcb);								 	// Range and frequency, 11.0hz
 			hmWriteReg(PWM_3_FREQ, 0xcb);								 	// Range and frequency, 11.0hz
 
			hmWriteReg(MIN_OFF_PWM_1_RAMP_RATE, 0x0b);
 			hmWriteReg(PWM_2_PWM_3_RAMP_RATE, 0x00);
 
	   		hmWriteReg(TACH_PWM_ASSOC, 0x24);								// PWM3-TACH3, PWM2-TACH2, PWM1-TACH1
 
	  		hmWriteReg(PWM_1_OPTION, (GRD0 | UPDT0 | SZEN));
	  		hmWriteReg(PWM_2_OPTION, (GRD0 | SZEN));
	  		hmWriteReg(PWM_3_OPTION, (GRD0 | SZEN));
 
	   		hmWriteReg(FANTACH_1_OPTION, FT_EDG1);
	   		hmWriteReg(FANTACH_2_OPTION, FT_EDG1);
	   		hmWriteReg(FANTACH_3_OPTION, FT_EDG1);
  		
	   		hmWriteReg(TACH_1_3_MODE, 0); 							// normal mode, no locked rotor mode

			hmWriteReg(EN_LSBS_FOR_AUTOFAN, 0);
 
			hmWriteReg(THERMTRIP_OUT_EN, 0x00);						// No Thermal trip outputs enabled
			hmWriteReg(THERMTRIP_ENABLE, 0x00);	
			hmWriteReg(THERMTRIP_CNTL, 0x01);						// Thermal trip output 
		
			hmWriteReg(SPECIAL_FUNCTION, (AVG_1 | AVG_2 | AVG_0 | INTEN));	// Interrupts enabled


			hmWriteReg(CONFIGURATION, CF_SUREN);					// Spin up reduction enabled
		}
			
		x = hmReadReg(READY_LOCK_START);
		x |= START;
		hmWriteReg(READY_LOCK_START, x);								// Start monitoring
		        
#ifdef DEBUG
		printf("hwmon_init: starting...");
		_outp(0x80, 0x44);
#endif
        // poll for voltage and temperature readings ready
 		while(!(hmReadReg(CONFIGURATION) & CF_TRDY)) {
			break;
		}

#ifdef DEBUG
		printf("hwmon_init: inited.");
		_outp(0x80, 0x45);
#endif

		running = TRUE;

	}
	return(1);
}

//
//-------------------------------------------------------------------
//
// register_dump - Dump the SCH3114 Hardware Monitoring Register Set
//
//--------------------------------------------------------------------
//

void register_dump(void)
{
	unsigned short c = 0;
	unsigned short reg = 0;
	unsigned char count = 0;
	
	c = sprintf(buff, "Hardware Monitoring Register Set:\n");
	while(reg <= 0x00ff) {
		if(!(count & 0x0f)) {
			c += sprintf(buff + c, "\n%02x: ", reg);
		}
		c += sprintf(buff + c, "%02x ", hmReadReg(reg));
		_outtext(buff);
		c = 0;
		count++;
		reg++;
	}
}

//
//------------------------------------------------------------------
//
// config_dump - Dump the SCH3114 Hardware Monitoring Configuration
//
//------------------------------------------------------------------
//

void config_dump(void)
{
	unsigned char x, lsb, msb, fanmode;
	unsigned short c, rpm;
		
	c = sprintf(buff, "Configuration:\n");

	// Ready Lock Start
	x = hmReadReg(READY_LOCK_START);
	c += sprintf(buff + c, "%s 0x%02x->",  "READY_LOCK_START:            ", x);
 	if(x & VBAT_MON) {
	 	c += sprintf(buff + c, "VBAT_MON ");
	}
	if(x & OVRID) {
	 	c += sprintf(buff + c, "OVRID ");
	}
 	if(x & READY) {
	 	c += sprintf(buff + c, "READY ");
	}
	if(x & LOCK) {
	 	c += sprintf(buff + c, "LOCK ");
	}
	if(x & START) {
	 	c += sprintf(buff + c, "START ");
	}
 	c += sprintf(buff + c, "\n");

	x = hmReadReg(SPECIAL_FUNCTION);
	c += sprintf(buff + c, "%s 0x%02x->",  "SPECIAL_FUNCTION:            ", x);
	if(x & INTEN) {
	 	c += sprintf(buff + c, "INTEN ");
	}
	if(x & MONITOR_MD) {
	 	c += sprintf(buff + c, "CYCLE ");
	} else {
	 	c += sprintf(buff + c, "CONTINUOUS ");
	}
 	c += sprintf(buff + c, "\n");

	// ThermTrip Control
	x = hmReadReg(THERMTRIP_CNTL);
	c += sprintf(buff + c, "%s 0x%02x->",  "THERMTRIP_CNTL:              ", x);
	if(x & 0x01) {
		c += sprintf(buff + c, "ON ");
	}
 	c += sprintf(buff + c, "\n");
	
	// ThermTrip Output Enable
	x = hmReadReg(THERMTRIP_OUT_EN);
	c += sprintf(buff + c, "%s 0x%02x->",  "THERMTRIP_OUT_EN:            ", x);
	if(x & AMBIENT_TRIP) {
		c += sprintf(buff + c, "AMBIENT ");
	}
	if(x & D1_TRIP) {
		c += sprintf(buff + c, "D1 ");
	}
	if(x & D2_TRIP) {
		c += sprintf(buff + c, "D2 ");
	}
	c += sprintf(buff + c, "\n");  	

	// ThermTrip Output Enable
	x = hmReadReg(THERMTRIP_ENABLE);
	c += sprintf(buff + c, "%s 0x%02x->",   "THERM_TRIP_ENABLE:           ", x);
	if(x & AMBIENT_TRIP) {
		c += sprintf(buff + c, "AMBIENT ");
	}
	if(x & D1_TRIP) {
		c += sprintf(buff + c, "D1 ");
	}
	if(x & D2_TRIP) {
		c += sprintf(buff + c, "D2 ");
	}
	c += sprintf(buff + c, "\n"); 
	
	// High, Low, ABS, Temps
	x = hmReadReg(D1_LOW_TEMP);
	c += sprintf(buff + c, "%s 0x%02x\n",   "D1_LOW_TEMP:                 ", x);
	x = hmReadReg(D1_HIGH_TEMP);
	c += sprintf(buff + c, "%s 0x%02x\n",   "D1_HIGH_TEMP:                ", x);
	x = hmReadReg(TOP_TEMP_REMOTE_DIODE_1);
	c += sprintf(buff + c, "%s 0x%02x\n",   "TOP_TEMP_REMOTE_DIODE_1:     ", x);
	x = hmReadReg(OFFSET_D1);
	c += sprintf(buff + c, "%s 0x%02x\n",   "OFFSET_D1:                   ", x);
	x = hmReadReg(THERMTRIP_TEMP_LIMIT_D1);
	c += sprintf(buff + c, "%s 0x%02x\n",   "THERMTRIP_TEMP_LIMIT_D1:     ", x);
	
	x = hmReadReg(AMBIENT_LOW_TEMP);
	c += sprintf(buff + c, "%s 0x%02x\n",   "AMBIENT_LOW_TEMP:            ", x);
	x = hmReadReg(AMBIENT_HIGH_TEMP);
	c += sprintf(buff + c, "%s 0x%02x\n",   "AMBIENT_HIGH_TEMP:           ", x);
	x = hmReadReg(TOP_TEMP_AMBIENT);
	c += sprintf(buff + c, "%s 0x%02x\n",   "TOP_TEMP_AMBIENT:            ", x);
	x = hmReadReg(OFFSET_AMBIENT);
	c += sprintf(buff + c, "%s 0x%02x\n",   "OFFSET_AMBIENT:              ", x);
	x = hmReadReg(THERMTRIP_TEMP_LIMIT_AMBIENT);
	c += sprintf(buff + c, "%s 0x%02x\n",   "THERMTRIP_TEMP_LIMIT_AMBIENT:", x);

	x = hmReadReg(D2_LOW_TEMP);
	c += sprintf(buff + c, "%s 0x%02x\n",   "D2_LOW_TEMP:                 ", x);
	x = hmReadReg(D2_HIGH_TEMP);
	c += sprintf(buff + c, "%s 0x%02x\n",   "D2_HIGH_TEMP:                ", x);
	x = hmReadReg(TOP_TEMP_REMOTE_DIODE_2);
	c += sprintf(buff + c, "%s 0x%02x\n",   "TOP_TEMP_REMOTE_DIODE_2:     ", x);
	x = hmReadReg(OFFSET_D2);
	c += sprintf(buff + c, "%s 0x%02x\n",   "OFFSET_D2:                   ", x);
	x = hmReadReg(THERMTRIP_TEMP_LIMIT_D2);
	c += sprintf(buff + c, "%s 0x%02x\n",   "THERMTRIP_TEMP_LIMIT_D2:     ", x);

//----------------------------------------------------------------------+----------------------------------------------------------------

	_outtext(buff);
 
    _settextposition((vc.numtextrows - 1), ((vc.numtextcols / 2) - (strlen("           <Press any key to continue>           ") / 2)));
	_outtext("           <Press any key to continue>           ");
	_getch();
 
	c = 0;
	_clearscreen(_GCLEARSCREEN);	// Clear the screen
 	_settextposition(1, 1);			// Start at the top left corner

//----------------------------------------------------------------------+----------------------------------------------------------------

	// Fantach minimum RPMs
	msb = hmReadReg(TACH_1_MIN_MSB);
	lsb = hmReadReg(TACH_1_MIN_LSB);
	rpm = ((msb << 8) | lsb);
	c += sprintf(buff + c, "%s 0x%04x\n",   "TACH_1_MIN RPM:       ", rpm);

	msb = hmReadReg(TACH_2_MIN_MSB);
	lsb = hmReadReg(TACH_2_MIN_LSB);
 	rpm = ((msb << 8) | lsb);
	c += sprintf(buff + c, "%s 0x%04x\n",   "TACH_2_MIN RPM:       ", rpm);

	msb = hmReadReg(TACH_3_MIN_MSB);
	lsb = hmReadReg(TACH_3_MIN_LSB);
 	rpm = ((msb << 8) | lsb);
 	c += sprintf(buff + c, "%s 0x%04x\n",   "TACH_3_MIN RPM:       ", rpm);

	// Fantach Options
	x = hmReadReg(FANTACH_1_OPTION);
	c += sprintf(buff + c, "%s 0x%02x->",   "FANTACH_1_OPTION:       ", x);
	if(x & FT_3EDG) {
		c += sprintf(buff + c, "FT_3EDG ");
	}
	if(x & FT_MODE) {
		c += sprintf(buff + c, "FT_MODE=mode 2 enhanced ");
	}
	if(x & FT_EDG1) {
		c += sprintf(buff + c, "FT_EDG1 ");
	}
	if(x & FT_EDG0) {
		c += sprintf(buff + c, "FT_EDG0 ");
	}
	if(x & FT_SLOW) {
		c += sprintf(buff + c, "FT_SLOW ");
	}
	c += sprintf(buff + c, "\n");
	
	x = hmReadReg(FANTACH_2_OPTION);
	c += sprintf(buff + c, "%s 0x%02x->",   "FANTACH_2_OPTION:       ", x);
 	if(x & FT_3EDG) {
		c += sprintf(buff + c, "FT_3EDG ");
	}
	if(x & FT_MODE) {
		c += sprintf(buff + c, "FT_MODE=mode 2 enhanced ");
	}
	if(x & FT_EDG1) {
		c += sprintf(buff + c, "FT_EDG1 ");
	}
	if(x & FT_EDG0) {
		c += sprintf(buff + c, "FT_EDG0 ");
	}
	if(x & FT_SLOW) {
		c += sprintf(buff + c, "FT_SLOW ");
	}
	c += sprintf(buff + c, "\n");

	
	x = hmReadReg(FANTACH_3_OPTION);
	c += sprintf(buff + c, "%s 0x%02x->",   "FANTACH_3_OPTION:       ", x);
	if(x & FT_3EDG) {
		c += sprintf(buff + c, "FT_3EDG ");
	}
	if(x & FT_MODE) {
		c += sprintf(buff + c, "FT_MODE=mode 2 enhanced ");
	}
	if(x & FT_EDG1) {
		c += sprintf(buff + c, "FT_EDG1 ");
	}
	if(x & FT_EDG0) {
		c += sprintf(buff + c, "FT_EDG0 ");
	}
	if(x & FT_SLOW) {
		c += sprintf(buff + c, "FT_SLOW ");
	}
	c += sprintf(buff + c, "\n");
	
	// PWM Freqs
	x = hmReadReg(PWM_1_FREQ);
	c += sprintf(buff + c, "%s 0x%02x\n",   "PWM_1_FREQ:             ", x);

	x = hmReadReg(PWM_2_FREQ);
	c += sprintf(buff + c, "%s 0x%02x\n",   "PWM_2_FREQ:             ", x);

	x = hmReadReg(PWM_3_FREQ);
	c += sprintf(buff + c, "%s 0x%02x\n",   "PWM_3_FREQ:             ", x);

	// PWM Configs
	x = hmReadReg(PWM_1_CONFIG);
	fanmode = (x & ZONE_MASK);
	c += sprintf(buff + c, "%s 0x%02x->",   "PWM_1_CONFIG:           ", x);
	c = show_fanmode(fanmode, c);
	if(x & INV) {
		c += sprintf(buff + c, "INV ");
	}
	if(x & SUEN) {
		c += sprintf(buff + c, "SUEN ");
	}
	c = show_spin((unsigned char)(x & SPIN_MASK), c);
	c += sprintf(buff + c, "\n");
	
	x = hmReadReg(PWM_2_CONFIG);
	fanmode = (x & ZONE_MASK);
	c += sprintf(buff + c, "%s 0x%02x->",   "PWM_2_CONFIG:           ", x);
	c = show_fanmode(fanmode, c);
	if(x & INV) {
		c += sprintf(buff + c, "INV ");
	}
	if(x & SUEN) {
		c += sprintf(buff + c, "SUEN ");
	}
	c = show_spin((unsigned char)(x & SPIN_MASK), c);
	c += sprintf(buff + c, "\n");

	x = hmReadReg(PWM_3_CONFIG);
	fanmode = (x & ZONE_MASK);
	c += sprintf(buff + c, "%s 0x%02x->",   "PWM_3_CONFIG:           ", x);
	c = show_fanmode(fanmode, c);
	if(x & INV) {
		c += sprintf(buff + c, "INV ");
	}
	if(x & SUEN) {
		c += sprintf(buff + c, "SUEN ");
	}
	c = show_spin((unsigned char)(x & SPIN_MASK), c);
	c += sprintf(buff + c, "\n");
	
	// PWM minimum duty cycle
	x = hmReadReg(PWM_1_MIN_DUTY);
	c += sprintf(buff + c, "%s 0x%02x\n",   "PWM_1_MIN_DUTY:         ", x);
	x = hmReadReg(PWM_2_MIN_DUTY);
	c += sprintf(buff + c, "%s 0x%02x\n",   "PWM_2_MIN_DUTY:         ", x);
	x = hmReadReg(PWM_3_MIN_DUTY);
	c += sprintf(buff + c, "%s 0x%02x\n",   "PWM_3_MIN_DUTY:         ", x);

	// PWM max duty cycle
	x = hmReadReg(PWM_1_MAX_DUTY);
	c += sprintf(buff + c, "%s 0x%02x\n",   "PWM_1_MAX_DUTY:         ", x);
	x = hmReadReg(PWM_2_MAX_DUTY);
	c += sprintf(buff + c, "%s 0x%02x\n",   "PWM_2_MAX_DUTY:         ", x);
	x = hmReadReg(PWM_3_MAX_DUTY);
	c += sprintf(buff + c, "%s 0x%02x\n",   "PWM_3_MAX_DUTY:         ", x);

	// PWM Options
	x = hmReadReg(PWM_1_OPTION);
	c += sprintf(buff + c, "%s 0x%02x->",   "PWM_1_OPTION:           ", x);
	if(x & OPP) {
		c += sprintf(buff + c, "OPP ");
	}
	if(x & GRD1) {
		c += sprintf(buff + c, "GRD1 ");
	}
	if(x & GRD0) {
		c += sprintf(buff + c, "GRD0 ");
	}
	if(x & SZEN) {
		c += sprintf(buff + c, "SZEN ");
	}
	if(x & UPDT1) {
		c += sprintf(buff + c, "UPDT1 ");
	}
	if(x & UPDT0) {
		c += sprintf(buff + c, "UPDT0 ");
	}
	c += sprintf(buff + c, "\n");

	x = hmReadReg(PWM_2_OPTION);
	c += sprintf(buff + c, "%s 0x%02x->",   "PWM_2_OPTION            ", x);
	if(x & OPP) {
		c += sprintf(buff + c, "OPP ");
	}
	if(x & GRD1) {
		c += sprintf(buff + c, "GRD1 ");
	}
	if(x & GRD0) {
		c += sprintf(buff + c, "GRD0 ");
	}
	if(x & SZEN) {
		c += sprintf(buff + c, "SZEN ");
	}
	if(x & UPDT1) {
		c += sprintf(buff + c, "UPDT1 ");
	}
	if(x & UPDT0) {
		c += sprintf(buff + c, "UPDT0 ");
	}
	c += sprintf(buff + c, "\n");

	x = hmReadReg(PWM_3_OPTION);
	c += sprintf(buff + c, "%s 0x%02x->",   "PWM_3_OPTION:           ", x);
	if(x & OPP) {
		c += sprintf(buff + c, "OPP ");
	}
	if(x & GRD1) {
		c += sprintf(buff + c, "GRD1 ");
	}
	if(x & GRD0) {
		c += sprintf(buff + c, "GRD0 ");
	}
	if(x & SZEN) {
		c += sprintf(buff + c, "SZEN ");
	}
	if(x & UPDT1) {
		c += sprintf(buff + c, "UPDT1 ");
	}
	if(x & UPDT0) {
		c += sprintf(buff + c, "UPDT0 ");
	}
	c += sprintf(buff + c, "\n");

	// Tach PWM ASSOC
	x = hmReadReg(TACH_PWM_ASSOC);
	c += sprintf(buff + c, "%s 0x%02x->",   "TACH_PWM_ASSOC:         ", x);

	c += sprintf(buff + c, "T3=");
	c = show_tach_pwm((unsigned char)((x & TACH3_PWM_ASSOC_MASK) >> TACH3_PWM_ASSOC_SHIFT), c);
	c += sprintf(buff + c, "T2=");
	c = show_tach_pwm((unsigned char)((x & TACH2_PWM_ASSOC_MASK) >> TACH2_PWM_ASSOC_SHIFT), c);
	c += sprintf(buff + c, "T1=");
	c = show_tach_pwm((unsigned char)(x & TACH1_PWM_ASSOC_MASK), c);
	c += sprintf(buff + c, "\n");

	// Tach 1_3 mode
	x = hmReadReg(TACH_1_3_MODE);
	c += sprintf(buff + c, "%s 0x%02x\n",   "TACH_1_3_MODE:          ", x);

	// Ramp Rate
	x = hmReadReg(MIN_OFF_PWM_1_RAMP_RATE);
	c += sprintf(buff + c, "%s 0x%02x->",   "MIN_OFF_PWM_1_RAMP_RATE:", x);
	if(x & OFF3) {
		c += sprintf(buff + c, "OFF3 ");
	}
	if(x & OFF2) {
		c += sprintf(buff + c, "OFF2 ");
	}
	if(x & OFF1) {
		c += sprintf(buff + c, "OFF1 ");
	}
	if(x & RR1E) {
		c += sprintf(buff + c, "PWM1_RAMP_RATE_ENABLE Rate=");
		c = show_ramp_rate((unsigned char)(x & PWM1_RAMPRATE_MASK), c);
	}
	c += sprintf(buff + c, "\n");
	
	x = hmReadReg(PWM_2_PWM_3_RAMP_RATE);
	c += sprintf(buff + c, "%s 0x%02x->",   "PWM_2_PWM_3_RAMP_RATE:  ", x);
	if(x & RR2E) {
		c += sprintf(buff + c, "PWM2_RAMP_RATE_ENABLE Rate=");
		c = show_ramp_rate((unsigned char)((x & PWM2_RAMPRATE_MASK) >> 4), c);
		c += sprintf(buff + c, " ");
 	}
	if(x & RR3E) {
		c += sprintf(buff + c, "PWM3_RAMP_RATE_ENABLE Rate=");
		c = show_ramp_rate((unsigned char)(x & PWM3_RAMPRATE_MASK), c);
	}
	c += sprintf(buff + c, "\n");
	
	// Enable LSBS for Autofan
	x = hmReadReg(EN_LSBS_FOR_AUTOFAN);
	c += sprintf(buff + c, "%s 0x%02x\n",   "EN_LSBS_FOR_AUTOFAN:    ", x);

 //---------------------------------------------------------------------------------------------------------------------------------------

	_outtext(buff);
 
    _settextposition((vc.numtextrows - 1), ((vc.numtextcols / 2) - (strlen("           <Press any key to continue>           ") / 2)));
	_outtext("           <Press any key to continue>           ");
	_getch();
 
	c = 0;
	_clearscreen(_GCLEARSCREEN);	// Clear the screen
 	_settextposition(1, 1);			// Start at the top left corner

//---------------------------------------------------------------------------------------------------------------------------------------

	// Zone low temp limits
	x = hmReadReg(ZONE_1_LOW_TEMP_LIMIT);
	c += sprintf(buff + c, "%s 0x%02x\n", "ZONE_1_LOW_TEMP_LIMIT:", x);
	x = hmReadReg(ZONE_2_LOW_TEMP_LIMIT);
	c += sprintf(buff + c, "%s 0x%02x\n", "ZONE_2_LOW_TEMP_LIMIT:", x);
	x = hmReadReg(ZONE_3_LOW_TEMP_LIMIT);
	c += sprintf(buff + c, "%s 0x%02x\n", "ZONE_3_LOW_TEMP_LIMIT:", x);
	
	// Zone absolute limits
	x = hmReadReg(ZONE_1_TEMP_ABS_LIMIT);
	c += sprintf(buff + c, "%s 0x%02x\n", "ZONE_1_TEMP_ABS_LIMIT:", x);
	x = hmReadReg(ZONE_2_TEMP_ABS_LIMIT);
	c += sprintf(buff + c, "%s 0x%02x\n", "ZONE_2_TEMP_ABS_LIMIT:", x);
	x = hmReadReg(ZONE_3_TEMP_ABS_LIMIT);
	c += sprintf(buff + c, "%s 0x%02x\n", "ZONE_3_TEMP_ABS_LIMIT:", x);

	// hysters
	x = hmReadReg(ZONE_1_ZONE_2_HYST);
	c += sprintf(buff + c, "%s 0x%02x->", "ZONE_1_ZONE_2_HYST:   ", x);
 	c += sprintf(buff + c, "ZONE_1_HYST=%2uC ZONE_2_HYST=%2uC\n", ((x & ZONE_1_HYST_MASK) >> 4), (x & ZONE_2_HYST_MASK));
	
	x = hmReadReg(ZONE_3_HYST);
	c += sprintf(buff + c, "%s 0x%02x->", "ZONE_3_HYST:          ", x);
	c += sprintf(buff + c, "ZONE_3_HYST=%2uC\n", ((x & ZONE_3_HYST_MASK) >> 4));
 
	// Interrupt Enables
	x = hmReadReg(INT_ENAB_TEMP);
	c += sprintf(buff + c, "%s 0x%02x->", "INT_ENAB_TEMP:        ", x);
	if(x & D2_INT_EN) {
		c += sprintf(buff + c, "D2_INT_EN ");
	}
	if(x & D1_INT_EN) {
		c += sprintf(buff + c, "D1_INT_EN ");
	}	
	if(x & AMBIENT_INT_EN) {
		c += sprintf(buff + c, "AMBIENT_INT_EN ");
	}
	if(x & TEMP_INT_EN) {
		c += sprintf(buff + c, "TEMP_INT_EN ");
	}
 	c += sprintf(buff + c, "\n");

	x = hmReadReg(INT_ENAB_FAN_TACHS);
	c += sprintf(buff + c, "%s 0x%02x->", "INT_ENAB_FAN_TACHS:   ", x);
	if(x & FANTACH_3_INT_EN) {
		c += sprintf(buff + c, "FANTACH_3_INT_EN ");
	}
	if(x & FANTACH_2_INT_EN) {
		c += sprintf(buff + c, "FANTACH_2_INT_EN ");
	}	
	if(x & FANTACH_1_INT_EN) {
		c += sprintf(buff + c, "FANTACH_1_INT_EN ");
	}
	if(x & FANTACH_INT_EN) {
		c += sprintf(buff + c, "FANTACH_INT_EN ");
	}
 	c += sprintf(buff + c, "\n");

	x = hmReadReg(INT_ENAB_VOLTAGES);
	c += sprintf(buff + c, "%s 0x%02x->", "INT_ENAB_VOLTAGES:    ", x);
	if(x & V_VCC_INT_EN) {
		c += sprintf(buff + c, "V_VCC_INT_EN ");
	}
	if(x & V_12_INT_EN) {
		c += sprintf(buff + c, "V_12_INT_EN ");
	}	
	if(x & V_5_INT_EN) {
		c += sprintf(buff + c, "V_5_INT_EN ");
	}
	if(x & V_VTR_INT_EN) {
		c += sprintf(buff + c, "V_VTR_INT_EN ");
	}
	if(x & V_VCCP_INT_EN) {
		c += sprintf(buff + c, "V_VCCP_INT_EN ");
	}
	if(x & V_2_5_INT_EN) {
		c += sprintf(buff + c, "V_2_5_INT_EN ");
	}
	if(x & V_VBAT_INT_EN) {
		c += sprintf(buff + c, "V_VBAT_INT_EN ");
	}	
	if(x & VOLT_INT_EN) {
		c += sprintf(buff + c, "VOLT_INT_EN ");
	}
	c += sprintf(buff + c, "\n");

	x = hmReadReg(TRIM_ADJ_EN);
	c += sprintf(buff + c, "%s 0x%02x\n", "TRIM_ADJ_EN:          ", x);
	
	x = hmReadReg(CONFIGURATION);
	c += sprintf(buff + c, "%s 0x%02x->", "CONFIGURATION:        ", x);
	if(x & CF_INIT) {
		c += sprintf(buff + c, "INIT ");
	}
	if(x & CF_SUREN) {
		c += sprintf(buff + c, "SUREN ");
	}
	if(x & CF_TRDY) {
		c += sprintf(buff + c, "TRDY ");
	}	
	if(x & CF_MON_DN) {
		c += sprintf(buff + c, "MON_DN ");
	}	
 	c += sprintf(buff + c, "\n");
	_outtext(buff);
}

//
//------------------------------------------------------------------
//
// show_fanmode - show what mode the fan is in
//
//------------------------------------------------------------------
//

unsigned short show_fanmode(unsigned char fanmode, unsigned short c)
{
	switch(fanmode) {
	case FAN_ON_ZONE_1_AUTO:
		c += sprintf(buff + c, "FAN_ON_ZONE_1_AUTO ");
		break;
	case FAN_ON_ZONE_2_AUTO:
		c += sprintf(buff + c, "FAN_ON_ZONE_2_AUTO ");
		break;
	case FAN_ON_ZONE_3_AUTO:
		c += sprintf(buff + c, "FAN_ON_ZONE_3_AUTO ");
		break;
	case FAN_ALWAYS_ON_FULL:
		c += sprintf(buff + c, "FAN_ALWAYS_ON_FULL ");
		break;
	case FAN_DISABLED:
		c += sprintf(buff + c, "FAN_DISABLED ");
		break;
	case FAN_HOTTEST_23:
		c += sprintf(buff + c, "FAN_HOTTEST_23 ");
		break;
	case FAN_HOTTEST_123:
		c += sprintf(buff + c, "FAN_HOTTEST_123 ");
		break;
	case FAN_MANUAL:
		c += sprintf(buff + c, "FAN_MANUAL ");
		break;
	}
	return(c);
}	

//
//------------------------------------------------------------------
//
// show_spin - show the spinup mode
//
//------------------------------------------------------------------
//

unsigned short show_spin(unsigned char spin, unsigned short c)
{
	switch(spin) {
	case SPIN_0MS:
		c += sprintf(buff + c, "0ms   ");
		break;
	case SPIN_100MS:
		c += sprintf(buff + c, "100ms ");
		break;
	case SPIN_250MS:
		c += sprintf(buff + c, "250ms ");
		break;
	case SPIN_400MS:
		c += sprintf(buff + c, "400ms ");
		break;
	case SPIN_700MS:
		c += sprintf(buff + c, "700ms ");
		break;
	case SPIN_1000MS:
		c += sprintf(buff + c, "1000ms ");
		break;
	case SPIN_2000MS:
		c += sprintf(buff + c, "2000ms ");
		break;
	case SPIN_4000MS:
		c += sprintf(buff + c, "4000ms ");
		break;
	}
	return(c);
}

//
//------------------------------------------------------------------
//
// show_tach_pwm - show which pwm go with what tachs
//
//------------------------------------------------------------------
//

unsigned short show_tach_pwm(unsigned char pwm, unsigned short c)
{
	switch(pwm) {
	case 0x00:
		c += sprintf(buff + c, "PWM1 ");
		break;
	case 0x01:
		c += sprintf(buff + c, "PWM2 ");
		break;
	case 0x02:
		c += sprintf(buff + c, "PWM3 ");
		break;
	case 0x03:
		c += sprintf(buff + c, "RSRVD");
		break;
	}
	return(c);
}

//
//------------------------------------------------------------------
//
// show_ramp_rate - show the fans ramp rate
//
//------------------------------------------------------------------
//

unsigned short show_ramp_rate(unsigned char ramp_rate, unsigned short c)
{
	switch(ramp_rate) {
	case 0x00:
		c += sprintf(buff + c, "206ms ");
		break;
	case 0x01:
		c += sprintf(buff + c, "104ms ");
		break;
	case 0x02:
		c += sprintf(buff + c, "69ms  ");
		break;
	case 0x03:
		c += sprintf(buff + c, "41ms  ");
		break;
	case 0x04:
		c += sprintf(buff + c, "26ms  ");
		break;
	case 0x05:
		c += sprintf(buff + c, "18ms  ");
		break;
	case 0x06:
		c += sprintf(buff + c, "10ms  ");
		break;
	case 0x07:
		c += sprintf(buff + c, "5ms   ");
		break;
	}
	return(c);
}

//
//--------------------------------------------------------------------------------
//
// statmon() - Show the "statuses" of the SCH3114 Hardware Monitoring Block
//
//--------------------------------------------------------------------------------
//

void statmon(void)
{
	short c;
	unsigned char x;

	c = sprintf(buff, "Status: ");

	x = hmReadReg(READY_LOCK_START);
	c += sprintf(buff + c, "(%s=", "READY_LOCK_START");
	c += sprintf(buff + c, "0x%02x)\n", x);

	// get and print ThermTrip status
	x = hmReadReg(THERMTRIP_STAT);
	if(reset_fault_statuses) {
		hmWriteReg(THERMTRIP_STAT, x);
	}
	c += sprintf(buff + c, "    %s=", "THERMTRIP_STAT        ");
	if(verbose) {
		c += sprintf(buff + c, "%02x=%02x->", THERMTRIP_STAT, x);
	}
	if(x & AMBIENT_TRIP) {
		c += sprintf(buff + c, "AMBIENT ");
	} else {
		c += sprintf(buff + c, "        ");
	}
	if(x & D1_TRIP) {
		c += sprintf(buff + c, "D1 ");
	} else {
		c += sprintf(buff + c, "   ");
	}
	if(x & D2_TRIP) {
		c += sprintf(buff + c, "D2");
	} else {
		c += sprintf(buff + c, "  ");
	}
	c += sprintf(buff + c, "\n");
 
 	// get and print Top Temp Exceeded Status
	x = hmReadReg(TOP_TEMP_EXCEEDED_STAT);
	if(reset_fault_statuses) {
		hmWriteReg(TOP_TEMP_EXCEEDED_STAT, x);
	}
 	c += sprintf(buff + c, "    %s=", "TOP_TEMP_EXCEEDED_STAT");
	if(verbose) {                  
		c += sprintf(buff + c, "%02x=%02x->", TOP_TEMP_EXCEEDED_STAT, x);
	}
	if(x & AMBIENT_TRIP) {
		c += sprintf(buff + c, "AMBIENT ");
	} else {
		c += sprintf(buff + c, "        ");
	}
	if(x & D1_TRIP) {
		c += sprintf(buff + c, "D1 ");
	} else {
		c += sprintf(buff + c, "   ");
	}
	if(x & D2_TRIP) {
		c += sprintf(buff + c, "D2 ");
	} else {
		c += sprintf(buff + c, "   ");
	}
	c += sprintf(buff + c, "\n");
  
  	// get and print Interrupt Status 1
	x = hmReadReg(INT_STAT_1);
 	if(reset_fault_statuses) {
		hmWriteReg(INT_STAT_1, x);
	}
	c += sprintf(buff + c, "    %s=", "INT_STAT_1            ");
	if(verbose) {
		c += sprintf(buff + c, "%02x=%02x->", INT_STAT_1, x);
	}
	if(x & IS1_INT23) {
		c += sprintf(buff + c, "INT23 ");
	} else {
		c += sprintf(buff + c, "      ");
	}
	if(x & IS1_D2) {
		c += sprintf(buff + c, "D2 ");
	} else {
		c += sprintf(buff + c, "   ");
	}
	if(x & IS1_AMB) {
		c += sprintf(buff + c, "AMBIENT ");
	} else {
		c += sprintf(buff + c, "        ");
	}
	if(x & IS1_D1) {
		c += sprintf(buff + c, "D1 ");
	} else {
		c += sprintf(buff + c, "   ");
	}
	if(x & IS1_5V) {
		c += sprintf(buff + c, "5V ");
	} else {
		c += sprintf(buff + c, "   ");
	} 
	if(x & IS1_VCC) {
		c += sprintf(buff + c, "VCC ");
	} else {
		c += sprintf(buff + c, "    ");
	}
	if(x & IS1_VCCP) {
		c += sprintf(buff + c, "VCCP ");
	} else {
		c += sprintf(buff + c, "     ");
	} 
	if(x & IS1_2_5V) {
		c += sprintf(buff + c, "2.5V ");
	} else {
		c += sprintf(buff + c, "     ");
	}
	c += sprintf(buff + c, "\n");
 
	// get and print Interrupt Status 2
	x = hmReadReg(INT_STAT_2);
 	if(reset_fault_statuses) {
		hmWriteReg(INT_STAT_2, x);
	}
	c += sprintf(buff + c, "    %s=", "INT_STAT_2            ");
	if(verbose) {
		c += sprintf(buff + c, "%02x=%02x->", INT_STAT_2, x);
	}
 	if(x & IS2_ERR2) {
		c += sprintf(buff + c, "ERR2 ");
	} else {
		c += sprintf(buff + c, "     ");
	}
	if(x & IS2_ERR1) {
		c += sprintf(buff + c, "ERR1 ");
	} else {
		c += sprintf(buff + c, "     ");
	}
	if(x & IS2_FANTACH3) {
		c += sprintf(buff + c, "FANTACH3 ");
	} else {
		c += sprintf(buff + c, "         ");
	}
	if(x & IS2_FANTACH2) {
		c += sprintf(buff + c, "FANTACH2 ");
	} else {
		c += sprintf(buff + c, "         ");
	}
	if(x & IS2_FANTACH1) {
		c += sprintf(buff + c, "FANTACH1 ");
	} else {
		c += sprintf(buff + c, "         ");
	} 
	if(x & IS2_12V) {
		c += sprintf(buff + c, "12V ");
	} else {
		c += sprintf(buff + c, "    ");
	}
	c += sprintf(buff + c, "\n");
 
 	// get and print Interrupt Status 3
	x = hmReadReg(INT_STAT_3);
 	if(reset_fault_statuses) {
		hmWriteReg(INT_STAT_3, x);
	}
	c += sprintf(buff + c, "    %s=", "INT_STAT_3            ");
	if(verbose) {
		c += sprintf(buff + c, "%02x=%02x->", INT_STAT_3, x);
	}
	if(x & IS3_VBAT) {
		c += sprintf(buff + c, "VBAT ");
	} else {
		c += sprintf(buff + c, "     ");
	}
	if(x & IS3_VTR) {
		c += sprintf(buff + c, "VTR ");
	} else {
		c += sprintf(buff + c, "    ");
	}
	c += sprintf(buff + c, "\n");
  
	// get and print PWM1 Current Duty Cycle
	x = hmReadReg(PWM_1_CUR_DUTY);
	c += sprintf(buff + c, "    %s=", "PWM_1_CUR_DUTY        ");
	if(verbose) {
		c += sprintf(buff + c, "%02x=%02x->", PWM_1_CUR_DUTY, x);
	}
	c += sprintf(buff + c, "%% %3.2f   \n\n", (float)(x * 0.392156863));  	// 0xff=100% 100/255=0.3921568627450980392156862745098
 
/*
	// get and print PWM2 Current Duty Cycle
	x = hmReadReg(PWM_2_CUR_DUTY);
	c += sprintf(buff + c, "    %s=", "PWM_2_CUR_DUTY        ");
	if(verbose) {
		c += sprintf(buff + c, "%02x=%02x->", PWM_2_CUR_DUTY, x);
	}
	c += sprintf(buff + c, "%% %3.2f   \n", (float)(x * 0.392156863));  	// 0xff=100% 100/255=0.3921568627450980392156862745098

	// get and print PWM3 Current Duty Cycle
	x = hmReadReg(PWM_3_CUR_DUTY);
	c += sprintf(buff + c, "    %s=", "PWM_3_CUR_DUTY        ");
	if(verbose) {
		c += sprintf(buff + c, "%02x=%02x->", PWM_3_CUR_DUTY, x);
	}
	c += sprintf(buff + c, "%% %3.2f   \n", (float)(x * 0.392156863));  	// 0xff=100% 100/255=0.3921568627450980392156862745098

*/
	_outtext(buff);
}

//
//--------------------------------------------------------------
//
// tempmon - Show Temperatures
//
//--------------------------------------------------------------
//

void tempmon(void)
{
	int ctemp, ftemp;
	short c;

	c = sprintf(buff, "Temperatures:\n");
	
	// get and print Remote Diode 1 temperature
	ctemp = hmReadReg(D1_TEMP_READING);
	ftemp = ((ctemp * 9) / 5) + 32;
	c += sprintf(buff + c, "    D1=      ");
	if(verbose) {
		c += sprintf(buff + c, "%02x=%02x->", D1_TEMP_READING, ctemp);
	}
	c += sprintf(buff + c, "%3d C, %3d F (Processor)\n", ctemp, ftemp);
	
	// get and print Ambient temperature
	ctemp = hmReadReg(AMBIENT_TEMP_READING);
	ftemp = ((ctemp * 9) / 5) + 32; 	
	c += sprintf(buff + c, "    Ambient= ");
	if(verbose) {
		c += sprintf(buff + c, "%02x=%02x->", AMBIENT_TEMP_READING, ctemp);
	}
	c += sprintf(buff + c, "%3d C, %3d F (Ambient)\n\n", ctemp, ftemp);
				
	// get and print Remote Diode 2 temperature (Note: The EBC-855 doesn't use this)
/*	ctemp = hmReadReg(D2_TEMP_READING);
	ftemp = ((ctemp * 9) / 5) + 32; 	
	c += sprintf(buff + c, "    D2=      ");
	if(verbose) {
		c += sprintf(buff + c, "%02x=%02x ->", D2_TEMP_READING, ctemp);
	}
	c += sprintf(buff + c, "%3d C, %3d F\n\n", ctemp, ftemp);
*/
	_outtext(buff);
}

//
//--------------------------------------------------------------
//
// voltmon - Show Voltages
//
//--------------------------------------------------------------
//

void voltmon(void)
{
	unsigned char raw_reg_read, rls_reg;
	float a, b, c, d;
	short cc;
			
	cc = sprintf(buff, "Voltages:\n");

	if(running) {
		rls_reg = hmReadReg(READY_LOCK_START);
		rls_reg |= VBAT_MON;						// set the VBAT_MON bit	
		hmWriteReg(READY_LOCK_START, rls_reg);

//		while(hmReadReg(READY_LOCK_START) & VBAT_MON) {		// Wait for VBAT register to be updated
//			;
//		}
		while(!(hmReadReg(CONFIGURATION) & CF_TRDY)) {		// Wait for TRDY register to be updated
			;
		}

	}
		
	raw_reg_read = hmReadReg(V_2_5_READING);
	a = (float)raw_reg_read;
	b = (float)((a * 2.50) * 10.00);
	c = (float)(b / 192.00);
	d = (float)(c / 10.00);
	cc += sprintf(buff + cc, "    ");
	if(verbose) {
		cc += sprintf(buff + cc, "%02x=%02x->", V_2_5_READING, raw_reg_read);
	}
	cc += sprintf(buff + cc, "2.5V =  %1.2fV\n", d);

	raw_reg_read = hmReadReg(V_1_5_READING);
	a = (float)raw_reg_read;
	b = (float)((a * 2.20) * 10.00);
	c = (float)(b / 192.00);
	d = (float)(c / 10.00);
	cc += sprintf(buff + cc, "    ");
	if(verbose) {
		cc += sprintf(buff + cc, "%02x=%02x->", V_1_5_READING, raw_reg_read);
	}
	cc += sprintf(buff + cc, "Vccp =  %1.2fV\n", d);
	
	raw_reg_read = hmReadReg(V_VCC_READING);
	a = (float)raw_reg_read;
	b = (float)((a * 3.30) * 10.00);
	c = (float)(b / 192.00);
	d = (float)(c / 10.00);
	cc += sprintf(buff + cc, "    ");
	if(verbose) {
		cc += sprintf(buff + cc, "%02x=%02x->", V_VCC_READING, raw_reg_read);
	}
	cc += sprintf(buff + cc, "VCC  =  %1.2fV\n", d);
	
	raw_reg_read = hmReadReg(V_5_READING);
	a = (float)raw_reg_read;
	b = (float)((a * 5.00) * 10.00);
	c = (float)(b / 192.00);
	d = (float)(c / 10.00);
	cc += sprintf(buff + cc, "    ");
	if(verbose) {
		cc += sprintf(buff + cc , "%02x=%02x->", V_5_READING, raw_reg_read);
	}
	cc += sprintf(buff + cc, "5V   =  %1.2fV\n", d);
	
	raw_reg_read = hmReadReg(V_12_READING);
	a = (float)raw_reg_read;
	b = (float)((a * 12.00) * 10.00);
	c = (float)(b / 192.00);
	d = (float)(c / 10.00);
	cc += sprintf(buff + cc, "    ");
	if(verbose) {
		cc += sprintf(buff + cc, "%02x=%02x->", V_12_READING, raw_reg_read);
	}
	cc += sprintf(buff + cc, "12V  = %2.2fV\n", d);
	
	raw_reg_read = hmReadReg(VTR_READING);
	a = (float)raw_reg_read;
	b = (float)((a * 3.30) * 10.00);
	c = (float)(b / 192.00);
	d = (float)(c / 10.00);
	cc += sprintf(buff + cc, "    ");
	if(verbose) {
		cc += sprintf(buff + cc, "%02x=%02x->", VTR_READING, raw_reg_read);
	}
	cc += sprintf(buff + cc, "VTR  =  %1.2fV\n", d);
	
	raw_reg_read = hmReadReg(VBAT_READING);
	a = (float)raw_reg_read;
	b = (float)((a * 3.00) * 10.00);
	c = (float)(b / 174.00);
	d = (float)(c / 10.00);
	cc += sprintf(buff + cc, "    ");
	if(verbose) {
		cc += sprintf(buff + cc, "%02x=%02x->", VBAT_READING, raw_reg_read);
	}
	cc += sprintf(buff + cc, "Vbat =  %1.2fV\n\n", d);
	_outtext(buff);
}

//
//--------------------------------------------------------------
//
// fanmon - Show fan speeds
//
//--------------------------------------------------------------
//

void fanmon(void)
{ 
	short c;
	
	c = sprintf(buff, "Fan Tachs:\n");

	c += sprintf(buff + c, "    Fan 1 = %5u RPM\n", read_fantach(1));

	// These are not used on the EBC-855

//	c += sprintf(buff + c, "    Fan 2 = %5u RPM\n", read_fantach(2));
//	c += sprintf(buff + c, "    Fan 3 = %5u RPM\n", read_fantach(3));

	sprintf(buff + c, "\n");
	
	_outtext(buff);
}


//
//--------------------------------------------------------------
//
// read_fantach - Read a specified fan tach
//
//--------------------------------------------------------------
//

unsigned int read_fantach(int fan)
{
	unsigned char reg, lsb, msb;
	unsigned short rpm, tacho;
	
	switch(fan) {
	case 1:
		reg = FANTACH_1_MSB;
		break;
	case 2:
		reg = FANTACH_2_MSB;
		break;
	case 3:
		reg = FANTACH_3_MSB;
		break;
	}

	msb = hmReadReg(reg);
	lsb = hmReadReg((unsigned char)(reg - 1));

	tacho = ((msb << 8) | lsb);

	if(tacho == SLOW_FAN) {
		// Slow fan
		rpm = 0;
	} else if(tacho == STOPPED_FAN) {
		// Locked rotor
		rpm = 0;
	} else {
		rpm = (unsigned short)((90000 * 60) / tacho);
	}
	return((unsigned int)rpm);
}

//
//--------------------------------------------------------------
//
// hmReadReg - Read a specified hm register
//
//--------------------------------------------------------------
//

unsigned char hmReadReg(unsigned char reg)
{
	_outp(SCH3114_HWMON_INDEX_REG, reg);
	return(_inp(SCH3114_HWMON_DATA_REG));
}

//
//--------------------------------------------------------------
//
// hmWriteReg - Write a specified hm register
//
//--------------------------------------------------------------
//

void hmWriteReg(unsigned char reg, unsigned char val)
{
	_outp(SCH3114_HWMON_INDEX_REG, reg);
	_outp(SCH3114_HWMON_DATA_REG, val);
}
		